<?php declare(strict_types=1);
/**
 * Banker
 *
 * A Caching library implementing psr/cache (PSR 6) and psr/simple-cache (PSR 16)
 *
 * PHP version 7.4
 *
 * @package     Banker
 * @author      Timothy J. Warren <tim@timshomepage.net>
 * @copyright   2016 - 2020  Timothy J. Warren
 * @license     http://www.opensource.org/licenses/mit-license.html  MIT License
 * @version     3.1.0
 * @link        https://git.timshomepage.net/timw4mail/banker
 */
namespace Aviat\Banker\Driver;

use Aviat\Banker\Exception\CacheException;
use Predis\Client;

/**
 * Redis cache backend
 */
class RedisDriver extends AbstractDriver {

	/**
	 * The object encapsulating the connection to the Redis server
	 *
	 * @var Client
	 */
	protected ?Client $conn;

	/**
	 * RedisDriver constructor.
	 *
	 * @codeCoverageIgnore
	 * @param array $config
	 * @param array $options - Predis library connection options
	 * @throws CacheException
	 */
	public function __construct(array $config = [], array $options = [])
	{
		if ( ! class_exists(Client::class))
		{
			throw new CacheException('The redis driver requires the predis/predis composer package to be installed.');
		}

		$this->conn = new Client($config, $options);
	}

	/**
	 * Disconnect from redis server
	 * @codeCoverageIgnore
	 */
	public function __destruct()
	{
		$this->conn->quit();
	}

	/**
	 * See if a key currently exists in the cache
	 *
	 * @param string $key
	 * @return bool
	 */
	public function exists(string $key): bool
	{
		return (bool) $this->conn->exists($key);
	}

	/**
	 * Get the value for the selected cache key
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function get(string $key)
	{
		$raw = $this->conn->get($key);
		return unserialize($raw);
	}

	/**
	 * Set a cached value
	 *
	 * @param string $key
	 * @param mixed $value
	 * @param int $expires
	 * @return bool
	 */
	public function set(string $key, $value, ?int $expires = 0): bool
	{
		$value = serialize($value);

		if ($expires !== 0)
		{
			return (bool) $this->conn->set($key, $value, 'EX', $expires);
		}

		return (bool)$this->conn->set($key, $value);
	}

	/**
	 * Remove an item from the cache
	 *
	 * @param string $key
	 * @return boolean
	 */
	public function delete(string $key): bool
	{
		return (bool) $this->conn->del([$key]);
	}

	/**
	 * Remove multiple items from the cache
	 *
	 * @param string[] $keys
	 * @return boolean
	 */
	public function deleteMultiple(array $keys = []): bool
	{
		$res = $this->conn->del(...$keys);
		return $res === count($keys);
	}

	/**
	 * Empty the cache
	 *
	 * @return boolean
	 */
	public function flush(): bool
	{
		return (bool) $this->conn->flushdb();
	}

	/**
	 * Set the expiration timestamp of a key
	 *
	 * @param string $key
	 * @param int $expires
	 * @return boolean
	 */
	public function expiresAt(string $key, int $expires): bool
	{
		return (bool) $this->conn->expireat($key, $expires);
	}
}